/*COMPILADOR C18 V.3.40. MPLAB IDE V.8.63
Programa que obtiene la corriente rms mediante el sensor ACS712 con un rango de 5 amperes de CA
mediante el promedio de la corriente medida en 100, 500 y 1000 muestras, a partir de esta corriente 
promedio se calcula la corriente rms y finalmente al suponer un voltaje rms de 117 V se obtiene el 
consumo en VA.

Las conexiones para el SISTEMA BOLT PIC18F2550 se muestran:


																     	J6
		
																	1 _____ RA4
		
				       __________________________________________   2 _____ RA5
				  	  |
		          	  |												3 _____ RC0
		          	  |															   SISTEMA BOLT 
		 		  	  |												4 _____ RC1
	                  |	          
				   ___|__________________________________________  	5 _____ +5V
		          |   |
				  |	  |	  _______________________________________	6 _____ GND
				  |   |   |
				  |	  |   |											
				 _|___|___|__									       AUX
				|Vcc Out Gnd |										
				|			 |
				|	ACS712	 |
				|			 |
				|			 |
				|		     |
				| (|)   (|)	 |
				|____________|
					|    |
				 ___|    |_________________________
				|								  |	
				|								  |
		Fuente (AC)								 (X)	Carga	
				|								  |
				|_________________________________|

*/
 
#include <p18cxxx.h>
#include <delays.h>		//librerías de retrasos
#include <stdio.h>		//librería sprintf()
#include <stdlib.h>		//librerías atoi(),atof()
#include <adc.h>		//librerías del ADC
#include "ADC-BOLT.h"	//librería Bolt-ADC-Canal-4
#include "18F2550BOLT.h"//librería universal Bolt


#define TIEMPO 1000		//Tiempo en ms de espera para la vusualizacion de Irms y el consumo en el LCD	

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code

int nMuestras = 500;	//numero de muestras n = [100, 500, 1000], puede ser diferente

				  //01234567 
const char mess0[]="Irms: ";
const char mess1[]=" A";

char ent[5];		//arreglo que contendra la parte entera de la corriente rms
char frac[5];		//arreglo que contendra la parte decimal de la corriente rms
char cons[5];		//arreglo que contrendra en consumo en VA

const char mess2[]="Consumo:";
const char mess3[]=" VA";

void main(void)
{
	float Irms, inst_current, av_current;		//Variables de punto flotante para la corriente rms,
												//la corriente instantanea y la corriente promedio
	int whole, decimal, i;						//Variable auxiliares para el manejo de numeros de punto
												//flotante y control del numero de muestras.

	init_bolt();								//Inicializa puertos, para utilizar el relevador, los leds
												//y los microswitches en el sistema Bolt
	open_bolt_adc();							//Inicializa el canal 4 del convertidor A/D en el sistema
												//Bolt 18F2550 
	InitLCD();									//Envia los comandos para inicializar el LCD. El cursor va 
												//a la posicion (0,0).
	
	CICLO:
		av_current = 0;							//Inicializa el valor de la corriente promedio
	
		for(i=0; i<nMuestras; i++)
		{
			inst_current = 0.0254*( bolt_adc() - 512 );						//Se obtiene la corriente instantanea
			if(inst_current < 0) inst_current = - inst_current;				//rectificacion de la componente de AC
			av_current = av_current + inst_current / ((float)nMuestras);	//corriente promedio de AC rectificada
		}
		
		Irms = 1.1107 * av_current;
		if(Irms < 0.1) Irms=0;			//Esta parte elimina el ruido intrinseco del sensor ACS712
										//ademas con la configuracion del sensor esta es la resolucion 
										//que el sensor el capaz de entregar en su salida.

		whole = Irms;					//parte entera
		decimal = (Irms-whole)*1000;	//parte decimal, se utiliza 10 para 1 lugar, 100 para dos lugares
										//significativos y asi sucesivamente.

		sprintf(ent,(const far rom char*)"%1d",whole);		//convierte a una cadena
		sprintf(frac,(const far rom char*)"%03d",decimal);	//convierte a una cadena

		ClearScreen();			//Limpia la pantalla del LCD

		PrintString(mess0);		//Imprime en el LCD el arreglo del argumento
		PrintString(ent);		
		PutLCD('.');			//Imprime en el LCD el caracter de punto decimal
	
		GotoXY(1,0);			//Coloca el cursor en la fila 1 columna 0
								//El LCD 16x1 que tiene el Sistema Bolt se tiene las siguientes fila y columnas
								//
								//(fila,columna)
								//
								//(0,0)(0,1)(0,2)(0,3)(0,4)(0,5)(0,6)(0,7)(1,0)(1,1)(1,2)(1,3)(1,4)(1,5)(1,6)(1,7)
								//
								//por lo que para pasar de la mitad del LCD se debe cambiar de fila, esto no 
								//implica que el buffer de cada fila solo sea de 8 caracteres.
		PrintString(frac);		//Imprime la parte fraccionaria
		PrintString(mess1);		

		delay_ms(TIEMPO);		

		sprintf(cons,(const far rom char*)"%d",(int)(Irms*117));	//suponiendo Vrms = 117 [V] se calcula el
																	//consumo en VA
		ClearScreen();							
		PrintString(mess2);
		GotoXY(1,1);
		PrintString(cons);
		PrintString(mess3);

		delay_ms(TIEMPO);
		
	goto CICLO;
} 

